package chembal;

import java.lang.*;
import java.util.*;

/**
 * Interface for things that hold atoms
 *
 * <p>
 * Title: ChemBal
 * </p>
 *
 * <p>
 * Description: Balances Chemical Equations
 * </p>
 *
 * <p>
 * Copyright: Copyright (c) 2003
 * </p>
 *
 * <p>
 * Company:
 * </p>
 *
 * @author Jeremy Drake
 * @version 1.0
 */
public interface AtomContainer
{
	/**
	 * Get the number of atoms named <code>atomName</code> in this object.
	 *
	 * @param atomName Name of atom we're looking for
	 *
	 * @return Number of atoms of type atomName in this object
	 */
	public int getAtomCount(String atomName);

	/**
	 * Returns true if this object contains an <code>atomName</code> atom,
	 * returns false otherwise
	 *
	 * @param atomName Name of atom we're looking for
	 *
	 * @return true if this object contains an <code>atomName</code> atom, false
	 * 				 otherwise
	 */
	public boolean containsAtom(String atomName);

	/**
	 * Constructs the container based on the string representation
	 *
	 * @param strRep The string representation of the container
	 */
	public void fromString(String strRep);

	/**
	 * Gets a list of the string representations of all atoms in this object
	 *
	 * @return List of string representations of atoms
	 */
	public Set getAtomSet();

    /**
     * Gets the molar mass of the atoms in this object
     *
     * @return Molar mass which is the sum of the molar masses of the atoms
     */
    public double getMolarMass();

	/**
	 * Gets the string representation of this object
	 *
	 * @return The string representation of this object
	 */
	public String toString();
}
