package chembal;

import java.lang.*;
import java.util.*;

/**
 * Base class for things that hold lists of AtomContainers
 *
 * <p>
 * Title: ChemBal
 * </p>
 *
 * <p>
 * Description: Balances Chemical Equations
 * </p>
 *
 * <p>
 * Copyright: Copyright (c) 2003
 * </p>
 *
 * <p>
 * Company:
 * </p>
 *
 * @author Jeremy Drake
 * @version 1.0
 */
public abstract class BaseListWrapper implements AtomContainer
{
	protected java.util.List acList;

	/**
	 * Creates a new BaseListWrapper object.
	 */
	public BaseListWrapper()
	{
		acList = new java.util.LinkedList();
	}

	/**
	 * Creates a new BaseListWrapper object.
	 *
	 * @param acList List of AtomContainers in the BaseListWrapper
	 */
	public BaseListWrapper(AtomContainer [] acList)
	{
		this.acList = new java.util.LinkedList();

		for (int i = 0; i < acList.length; i++)
		{
			this.acList.add(i, acList[i]);
		}
	}

	/**
	 * Gets an iterator to the AtomContainer list that comprises the object
	 *
	 * @return Iterator of the element list
	 */
	public java.util.Iterator getIterator()
	{
		return acList.iterator();
	}

	/**
	 * Adds an <code>AtomContainer</code> to the object
	 *
	 * @param ac The<code>AtomContainer</code> to add
	 */
	public void addAtomContainer(AtomContainer ac)
	{
		this.acList.add(ac);
	}

	/**
	 * Get the number of atoms named <code>atomName</code> in this object. Since
	 *
	 * @param atomName Name of atom we're looking for
	 *
	 * @return Number of atoms of type atomName in this object
	 */
	public int getAtomCount(String atomName)
	{
		int retval = 0;

		for (Iterator i = acList.iterator(); i.hasNext();)
		{
			retval += ((AtomContainer) (i.next())).getAtomCount(atomName);
		}

		return retval;
	}

	/**
	 * Returns true if this object contains an <code>atomName</code> atom,
	 * returns false otherwise
	 *
	 * @param atomName Name of atom we're looking for
	 *
	 * @return true if this object contains an <code>atomName</code> atom, false
	 * 				 otherwise
	 */
	public boolean containsAtom(String atomName)
	{
		return this.getAtomCount(atomName) != 0;
	}

	/**
	 * Returns a <code>Set</code> containing the string representations of all
	 * atoms contained in this object
	 *
	 * @return The <code>Set</code> of all atoms in this object, in string form
	 */
	public Set getAtomSet()
	{
		Set anSet = new HashSet();
		Iterator it = this.getIterator();
		AtomContainer ac;

		while (it.hasNext())
		{
			ac = (AtomContainer) (it.next());
			anSet.addAll(ac.getAtomSet());
		}

		return anSet;
	}

    public double getMolarMass()
    {
        double mm=0;
        Iterator it = this.getIterator();

        while (it.hasNext())
        {
            AtomContainer e = (AtomContainer) (it.next());
            mm += e.getMolarMass();
        }

        return mm;
    }

}
