package chembal;

import java.lang.*;
import java.util.*;

/**
 * Container that represents one side of the equation
 *
 * <p>
 * Title: ChemBal
 * </p>
 *
 * <p>
 * Description: Balances Chemical Equations
 * </p>
 *
 * <p>
 * Copyright: Copyright (c) 2003
 * </p>
 *
 * <p>
 * Company:
 * </p>
 *
 * @author Jeremy Drake
 * @version 1.0
 */
public class EquationSide extends BaseListWrapper
{
	/**
	 * Creates a new EquationSide object.
	 */
	public EquationSide()
	{
		super();
	}

	/**
	 * Creates a new EquationSide object.
	 *
	 * @param mcList List of molecules in this side of the equation
	 */
	public EquationSide(Molecule [] mcList)
	{
		super(mcList);
	}

	/**
	 * Creates a new EquationSide object.
	 *
	 * @param strSide String representation of the <code>EquationSide</code> to
	 * 				construct
	 */
	public EquationSide(String strSide)
	{
		this();
		this.fromString(strSide);
	}

	/**
	 * Generate an <code>EquationSide</code> from a string representation
	 *
	 * @param strSide String representation of the <code>EquationSide</code> to
	 * 				generate
	 */
	public void fromString(String strSide)
	{
		StringTokenizer st = new StringTokenizer(strSide, " +", false);

		while (st.hasMoreTokens())
		{
			String t = st.nextToken();
			this.addMolecule(new Molecule(t));
		}
	}

	/**
	 * Add a <code>Molecule</code> to this side of the equation
	 *
	 * @param m <code>Molecule</code> to add
	 */
	public void addMolecule(Molecule m)
	{
		this.addAtomContainer(m);
	}

    public int numMolecules()
    {
        return this.acList.size();
    }

	/**
	 * Make a list of all molecules contained in this obj that contain the given
	 * atom
	 *
	 * @param atomName name of the atom to look for
	 *
	 * @return List of all molecules containing the given atom
	 */
	public List moleculesContainingAtom(String atomName)
	{
		List retval = new LinkedList();
		Iterator it = this.getIterator();

		while (it.hasNext())
		{
			Molecule mol = (Molecule) (it.next());

			if (mol.containsAtom(atomName))
			{
				retval.add(mol);
			}
		}

		return retval;
	}

	/**
	 * Gets the string representation of this object
	 *
	 * @return The string representation of this object
	 */
	public String toString()
	{
		boolean prePlus = false;
		String retval = "";
		Iterator it = this.getIterator();

		while (it.hasNext())
		{
			Molecule mol = (Molecule) (it.next());

			if (prePlus)
			{
				retval += "+";
			}
			else
			{
				prePlus = true;
			}

			retval += mol.toString();
		}

		return retval;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param args DOCUMENT ME!
	 */
	public static void main(String [] args)
	{
		EquationSide equationSide1 = new EquationSide();
	}
}
