package chembal;

import java.lang.*;
import java.util.*;
import java.util.regex.*;

/**
 * Class that represents one "molecule"
 *
 * <p>
 * Title: ChemBal
 * </p>
 *
 * <p>
 * Description: Balances Chemical Equations
 * </p>
 *
 * <p>
 * Copyright: Copyright (c) 2003
 * </p>
 *
 * <p>
 * Company:
 * </p>
 *
 * @author Jeremy Drake
 * @version 1.0
 */
public class Molecule extends BaseListWrapper
{
	private int coefficient;
    private boolean isPolyIon;

	/**
	 * Creates a new Molecule object.
	 */
	public Molecule()
	{
		super();
		coefficient = 1;
        isPolyIon=false;
	}

	/**
	 * Creates a new Molecule object.
	 *
	 * @param elList List of elements in the Molecule
	 */
	public Molecule(Element [] elList)
	{
		super(elList);
		coefficient = 1;
        isPolyIon=false;
	}

	/**
	 * Creates a new Molecule object.
	 *
	 * @param strMol String representation of the <code>Molecule</code> to
	 * 				construct
	 */
	public Molecule(String strMol)
	{
		this();
		this.fromString(strMol);
	}

	/**
	 * Generate a <code>Molecule</code> from a string representation
	 *
	 * @param strMol String representation of the <code>Molecule</code> to
	 * 				generate
	 */
	public void fromString(String strMol)
	{
        Pattern co = Pattern.compile("^(\\d+)\\(?[A-Z][a-z]*");
        Matcher ma = co.matcher(strMol);
        if(ma.find())
            this.coefficient = Integer.parseInt(ma.group(1));
        else
            this.coefficient = 1;

        co = Pattern.compile("\\(");
        ma = co.matcher(strMol);

        if(ma.find())
        {
            // we're screwing up the order -- but that should be ok
            co = Pattern.compile("\\((.*?)\\)(\\d*)");
            ma = co.matcher(strMol);
            while(ma.find())
            {
                Molecule mol = new Molecule(ma.group(1));
                System.out.println(ma.group(2));
                if(ma.group(2) != null && !ma.group(2).equals(""))
                {
                    mol.coefficient=Integer.parseInt(ma.group(2));
                }
                mol.isPolyIon=true;
                this.addAtomContainer(mol);
            }

            // Hope we didn't step on anybody's toes!
            strMol=ma.replaceAll("");
        }

		Pattern p = Pattern.compile("[A-Z][a-z]*\\d*");
		Matcher m = p.matcher(strMol);

		while (m.find())
		{
			this.addElement(new Element(m.group(0)));
		}
	}

	/**
	 * Gets the coefficient associated with this molecule
	 *
	 * @return The coefficient
	 */
	public int getCoefficient()
	{
		return this.coefficient;
	}

	/**
	 * Sets the coefficient
	 *
	 * @param c What the coefficient is set to
	 */
	public void setCoefficient(int c)
	{
		coefficient = c;
	}

	/**
	 * Increment the coefficient, returning the new value
	 *
	 * @return The new value of the coefficient
	 */
	public int incCoefficient()
	{
		return ++coefficient;
	}

	/**
	 * Adds an <code>Element</code> to the molecule
	 *
	 * @param e The <code>Element</code> to add
	 */
	public void addElement(Element e)
	{
		this.addAtomContainer(e);
	}

	/**
	 * Get the number of atoms named <code>atomName</code> in this object. Since
	 * this is a <code>Molecule</code> object, this will return the coefficient
	 * the sum of the return value of .getAtomCount(atomName) of each of the
	 * elements in elementList
	 *
	 * @param atomName Name of atom we're looking for
	 *
	 * @return Number of atoms of type atomName in this object
	 */
	public int getAtomCount(String atomName)
	{
		int retval = super.getAtomCount(atomName);
		retval *= this.coefficient;

		return retval;
	}

        /**
         * Gets the string representation of this object
         *
         * @return The string representation of this object
         */
	public String toString()
	{
		String retval = "";

		if (this.coefficient > 1 && !this.isPolyIon)
		{
			retval += Integer.toString(this.coefficient);
		}else if(this.isPolyIon)
        {
            retval += "(";
        }

		Iterator it = this.getIterator();

		while (it.hasNext())
		{
			AtomContainer e = (AtomContainer) (it.next());
			retval += e.toString();
		}

        if(this.isPolyIon)
        {
            retval += ")";
            if (this.coefficient > 1)
            {
                retval += Integer.toString(this.coefficient);
            }
        }

		return retval;
	}

    public double getMolarMass()
    {
        double mm=0;
        Iterator it = this.getIterator();

        while (it.hasNext())
        {
            AtomContainer e = (AtomContainer) (it.next());
            mm += e.getMolarMass();
        }

        return mm * this.coefficient;
    }

	/**
	 * DOCUMENT ME!
	 *
	 * @param args DOCUMENT ME!
	 */
	public static void main(String [] args)
	{
		Molecule molecule1 = new Molecule();
	}
}
